/* c8 ignore start */
import type { ConfigType } from '@nestjs/config';
import type { MicroserviceOptions, NatsOptions } from '@nestjs/microservices';

import { Logger, VersioningType } from '@nestjs/common';
import { NestFactory } from '@nestjs/core';
import { Transport } from '@nestjs/microservices';
import { DocumentBuilder, SwaggerModule } from '@nestjs/swagger';

import { Application } from './application.js';
import { httpConfig } from './config/http.config.js';
import { natsConfig } from './config/nats.config.js';

const app = await NestFactory.create(Application);
app.enableCors();

const { url, user, password } = app.get(natsConfig.KEY) as ConfigType<
  typeof natsConfig
>;

const microserviceOptions: Required<NatsOptions> = {
  transport: Transport.NATS,
  options: {
    servers: [url],
  },
};

if (user && password) {
  microserviceOptions.options.user = user;
  microserviceOptions.options.pass = password;
}

app.connectMicroservice<MicroserviceOptions>(microserviceOptions);

app.enableVersioning({
  defaultVersion: ['1'],
  type: VersioningType.URI,
});

const swaggerConfig = new DocumentBuilder()
  .setTitle('Gaia-X OCM Proof Manager API')
  .setDescription('API documentation for Gaia-X OCM Proof Manager')
  .setVersion('1.0')
  .build();

const document = SwaggerModule.createDocument(app, swaggerConfig);

SwaggerModule.setup('/swagger', app, document);
await app.startAllMicroservices();

const { host, port } = app.get(httpConfig.KEY) as ConfigType<typeof httpConfig>;
await app.listen(port as number, host as string);

Logger.log(`Application is running on: ${await app.getUrl()}`);
/* c8 ignore stop */
