import type { MicroserviceOptions, NatsOptions } from '@nestjs/microservices';

import { Logger } from '@nestjs/common';
import { NestFactory } from '@nestjs/core';
import { Transport } from '@nestjs/microservices';

import { AppModule } from './app.module.js';
import { config } from './config/config.js';

const app = await NestFactory.create(AppModule);

const { url, user, password } = config().nats;
const options: Required<NatsOptions> = {
  transport: Transport.NATS,
  options: {},
};

if (url) {
  options.options.servers = [url];
}

if (user && password) {
  options.options.user = user;
  options.options.pass = password;
}

app.connectMicroservice<MicroserviceOptions>(options);

const port = config().port;
await app.listen(port as number);

Logger.log(`Application is running on: ${await app.getUrl()}`);
