import type { EventTenantsCreateInput } from '../apps/shared/dist/index.js';

import { Logger, Module } from '@nestjs/common';
import { NestFactory } from '@nestjs/core';
import {
  ClientsModule,
  Transport,
  type ClientProxy,
} from '@nestjs/microservices';
import { randomBytes } from 'node:crypto';
import { firstValueFrom } from 'rxjs';

import { EventTenantsCreate } from '../apps/shared/src/events/tenantEvents.js';

// These values are from the docker-compose.yml file
const NATS_URL = process.env.NATS_URL || 'nats://localhost:4222';
const NATS_USER = process.env.NATS_USER || 'nats_user';
const NATS_PASSWORD =
  process.env.NATS_PASSWORD || 'Rw+dYIymAQm9H6ELLNwSuGo1812jqQ==';

@Module({
  imports: [
    ClientsModule.register([
      {
        transport: Transport.NATS,
        name: 'TENANTS_CLIENT_SERVICE',
        options: {
          url: NATS_URL,
          user: NATS_USER,
          pass: NATS_PASSWORD,
        },
      },
    ]),
  ],
})
class Application {}

void (async () => {
  const app = await NestFactory.createApplicationContext(Application);
  const client: ClientProxy = app.get('TENANTS_CLIENT_SERVICE');

  await client.connect();

  const tenantLabel = process.argv[2] || "tenant_" + randomBytes(4).toString('hex');
  const response$ = client.send<EventTenantsCreate, EventTenantsCreateInput>(
    EventTenantsCreate.token,
    {
      label: tenantLabel,
    },
  );

  const response = await firstValueFrom(response$);

  Logger.log(`Tenant "${tenantLabel}" created with id: ${response.data.id}`);
  // Logger.debug(response);

  await app.close();
})();
